'----------------------------------------------------------------------------------
' Program:       SRM9000 Radio Dispatcher
' Module:        Global
' Description:   This module defines global variables and constants.
'
'
'
' This software is provided as Freeware by TMC Radio Pty Ltd.
' This software and information is provided 'As Is' without warranty of any
' kind, either expressed or implied. 
'
'----------------------------------------------------------------------------------

Module [Global]

    Public PhbkList(100, 3) As String
    Public StatusList(32, 2) As String
    Public AutoPollRequestList(102) As String
    Public PollRequestList(32, 8) As String      'Poll requests from AVL Mapper
    Public NumberOfPhbkEntries As Integer
    Public NumberOfStatusEntries As Integer
    Public NumberOfPollRequests As Integer
    Public CurrentAutoPollListPtr As Integer

    'Filenames
    Public PhbkFileName As String = ".\Phonebook.txt"
    Public StatusFileName As String = ".\Status.txt"
    Public LogFileName As String = ".\Log.txt"
    Public SetupFileName As String = ".\Setup.ini"
    Public AvlOptionsFileName As String = ".\Avl.ini"
    Public GpsDataFileName As String = ".\RadioGpsData.txt"
    Public PollReqFileName As String = ".\PollRequest.txt"

    'Dispatcher Radio's Trunk Identity
    Public OwnPrefix As Integer
    Public OwnIdent As Integer
    Public OwnIBI As Integer
    Public OwnFleetSize As Integer
    Public OwnHighestIdent As Integer
    Public OwnGrpPrefix As Integer
    Public OwnGrpIdent As Integer
    Public OwnGBI As Integer
    Public OwnGrpFleetSize As Integer
    Public OwnGrpHighestIdent As Integer

    Public SerialLink As Boolean = True                 'True  = LinkLost
    Public TrunkServiceIndicator As Boolean = False     'False = Out of Service
    Public TrunkBusyIndicator As Boolean = False        'False = Not Busy
    Public TrunkTransmitIndicator As Boolean = False    'False = Not Transmitting
    Public TrunkNpdCall As Boolean = False              'False = NPD call has not been setup 

    Public Structure TXMESSAGEPARAMETERS
        Public Destination As String
        Public MessageFormat As String
        Public AlertTone As String
        Public Status As String
        Public JobNumber As String
        Public Message As String
        Public TrunkNetworkResponse As String
    End Structure

    Public Structure PROGRAMSETUPOPTIONS
        Public ComPort As Integer
        Public RadioMode As Integer
        Public DebugMode As Integer
        Public PollRequestStatus As Integer

    End Structure

    Public Structure PROGRAMAVLOPTIONS
        Public SimocoAVL As Boolean
        Public AutoPoll As Boolean
        Public ShowPollRequests As Boolean
        Public ShowGpsResponse As Boolean
        Public PollTime As Integer
        Public Spare1 As Integer
        Public Spare2 As Integer
        Public Spare3 As Integer

    End Structure

    Public Structure AVLTABLE                           'Received GPS data to place in AVL database
        Public RadioID As Integer                       'Numeric identity of vehicle
        Public Status As Integer                        'Status Number  
        Public JobNumber As String                      '8 character Job string 
        Public FleetNumber As Integer                   'Fleet Identity
        Public AppDestination As Integer                'Specifies which application can use this data.
        Public GPS_TimeOfFix As Integer                 'Seconds Past Midnight
        Public GPS_DayOfFix As Integer                  'Days since 1st Jan 2000
        Public GPS_Latitude As Integer                  '0 - 18000000 Units of 10E-5 degrees
        Public GPS_Longitude As Integer                 '0 - 36000000 Units of 10E-5 degrees
        Public GPS_Speed As Double                      'km/h
        Public GPS_Heading As Integer                   '0 - 259 degrees 
        Public GPS_AgeOfData As Integer                 '0=NA, 1=>10sec, 2=Current
        Public GPS_Error As Integer
        Public Data As String                           'Optional Data

    End Structure

    Public SetupOptions As PROGRAMSETUPOPTIONS
    Public AvlOptions As PROGRAMAVLOPTIONS
    Public TmcAvlTable As AVLTABLE

    Public Const PMR As Integer = 0
    Public Const TRUNK As Integer = 1
    Public Const DEBUG_OFF As Integer = 0
    Public Const DEBUG_ON As Integer = 1
    Public Const LINK_ESTABLISHED As Boolean = False
    Public Const LINK_LOST As Boolean = True
    Public Const IN_SERVICE As Boolean = True
    Public Const OUT_OF_SERVICE As Boolean = False
    Public Const NOT_BUSY As Boolean = False
    Public Const BUSY As Boolean = True
    Public Const RECEIVE As Boolean = False
    Public Const TRANSMIT As Boolean = True
    Public Const NPD_OFF As Boolean = False
    Public Const NPD_ON As Boolean = True

    Public Const AUTOPOLL_OFF As Boolean = False
    Public Const AUTOPOLL_ON As Boolean = True
    Public Const AUTOPOLL_LISTSIZE As Integer = 50

    Public Const COL_TIME As Integer = 0
    Public Const COL_TOFROM As Integer = 1
    Public Const COL_NAME As Integer = 2
    Public Const COL_FLAGS As Integer = 3
    Public Const COL_MESSAGE As Integer = 4
    Public Const COL_JOBNO As Integer = 5

    Public Const PHBK_NAME As Integer = 0
    Public Const PHBK_NUMBER As Integer = 1
    Public Const PHBK_RADIOTYPE As Integer = 2

    Public Const STATUS_NUMBER As Integer = 0
    Public Const STATUS_NAME As Integer = 1

    Public Const SMALL_FLEET As Integer = 0
    Public Const LARGE_FLEET As Integer = 1

End Module
