//---------------------------------------------------------------------------
//   File:        RXFUNC_UNIT.CPP
//
//   Description: Each serial messages received from the 9020, 9025 or 9030
//                Control head, contains a SubType Command.
//                The action to be performed for each Command is defined in
//                this file.
//                
//
//   Variations:  Ver   Date         FR     Description
//                -----------------------------------------------------------
//                V1.00 04/05/00     FRxxx  File created.
//
//
//   Public Functions:  
//                Process_VolumeADC()
//                Process_9030KeyEvent()
//                Process_ConsoleType()
//
//
//
//   COPYRIGHT (C) 2000 SIMOCO Pacific Pty Ltd.
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "RxFunc_Unit.h"
#include "TxFunc_Unit.h"
#include "DAT_Unit.h"
#include "MainForm.h"
#include "stdio.h"
#include "stdlib.h"
#include "CFG_Form.h"
#include "PMR_Form.h"

void  Process_VolumeADC(unsigned char *);
void  Process_9030KeyEvent(unsigned char *);
void  Process_ConsoleType(unsigned char *);
void  Process_IncomingCmd(unsigned char *);
extern void  DisplayRxSerialData(char *);    //CFG_FORM.CPP
extern void  DisplayPmrRxSerialData(char *); //PMR_FORM.CPP
void  GetCmdDefinition(unsigned char *, int );
void  GetPmrCmdDefinition(unsigned char *, int );
extern int serialProtocol; //V1.04
                         
int   ContrastValue=7;
                            
//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------------------------------------------------------
//   Function:     Process_VolumeADC()
//
//   Description:  The rotary volume knob on the SRM9030 head has been
//                 changed, so update the display.
//                 
//
//
//---------------------------------------------------------------------------
void  Process_VolumeADC(unsigned char *VolumeADCPtr)
{
    //Volume
};

//---------------------------------------------------------------------------
//   Function:     Process_9030KeyEvent()
//
//   Description:  A key event serial message has been received so
//                 find out which key it is and its state, then update
//                 the global ButtonState table.
//
//
//---------------------------------------------------------------------------
void  Process_9030KeyEvent(unsigned char *KeyEventPtr)
{
    //Display 9030
                              
};

//---------------------------------------------------------------------------
//   Function:     Process_ConsoleType()
//
//   Description:  Message specifiying whether the control head (Console) is
//                 a 9010, 9020, 9025, or 9030 head.
//                 Once the head type is determined, select the correct form
//                 to display.
//
//---------------------------------------------------------------------------
void  Process_ConsoleType(unsigned char *ConsoleTypePtr)
{
    int  Console;

    Console = ConsoleTypePtr[0];
    if(Console == 0x03)
    {
         //Show the T9030 Form
         HeadType = HEAD_9030;
    }        
              
   //Clear Entire Display
   SetDisplayIcon(0, 1, 0, 16);
   //Initialise contrast
   SetContrast(7);
   ContrastValue = 7;
   //Clear Button Status Array
   for (int j=0; j<40; j++)
   {
       ButtonState[j] = 0;
   }   
   //Initialise Head specific parameters.
   if(HeadType == HEAD_9030)
   {
       //9030 Head
   }
                                  

};

//---------------------------------------------------------------------------
//   Function:     Process_IncomingCmd()
//
//   Description:  Mobile has received a Serial Command, so
//                 work out which one it is and display it in
//                 the list box.
//
//
//---------------------------------------------------------------------------
void  Process_IncomingCmd(unsigned char *MPT1327CmdPtr)
{

    char          buffer[1024];
    unsigned char highNibble;
    unsigned char lowNibble;
    int           PacketLength;

    int      i, j, k;
    int      MinimumMsgLength, MPT1327CmdLength;

    //Get the length of data
    PacketLength = MPT1327CmdPtr[0];

    if(serialProtocol == 0) //V1.04
      k = 5;
    else
      k = 1;

    if(PacketLength < k) //V1.04
       return;
                   
    j=0;
    for(int i=k; i<=PacketLength; i++) //V1.04
    {
         highNibble = (MPT1327CmdPtr[i] & 0xF0) >> 4;
         lowNibble  = MPT1327CmdPtr[i] & 0x0F;

         if(highNibble <= 0x09)
            buffer[j++] = 48 + highNibble;
         else
            buffer[j++] = 65 + highNibble - 10;

         if(lowNibble <= 0x09)
            buffer[j++] = 48 + lowNibble;
         else
            buffer[j++] = 65 + lowNibble - 10;
            
         //Display Space between each "byte"  
         buffer[j++] = 32;  
          
    }
       
    //Add terminating character
    buffer[j] = '\0';
  
    //Write formatted line to Memo
    if(TrunkPmrMode == 0)
    {
       if(DebugMode == 1)
          DisplayRxSerialData(buffer);
    }
    else
    {
       if(DebugMode == 1)
          DisplayPmrRxSerialData(buffer);
    }

    //------------------------------------------
    if (serialProtocol == 0) //V1.04
    {
         MPT1327CmdLength = MPT1327CmdPtr[0] - 4;
         if(SETUP.Mode == MODE_TRUNK)
            GetCmdDefinition(&MPT1327CmdPtr[5], MPT1327CmdLength);
         else if(SETUP.Mode == MODE_PMR)
            GetPmrCmdDefinition(&MPT1327CmdPtr[5], MPT1327CmdLength);
     }
     else
     {
         MPT1327CmdLength = MPT1327CmdPtr[0];
         if(SETUP.Mode == MODE_TRUNK)
            GetCmdDefinition(&MPT1327CmdPtr[1], MPT1327CmdLength);
         else if(SETUP.Mode == MODE_PMR)
            GetPmrCmdDefinition(&MPT1327CmdPtr[1], MPT1327CmdLength);
     }
};
    
//---------------------------------------------------------------------------
//
//  Get TRUNK Command definition.
//
//
//
//---------------------------------------------------------------------------
void  GetCmdDefinition(unsigned char *MPT1327SerialPtr, int CmdLength)
{

    char          buffer[1024];
    unsigned char highNibble;
    unsigned char lowNibble;
    int           PacketLength;

    int      MyIBI, Unit, Status;
    char     UnitStr[10];
    char     StatusStr[10];
    char     GPSStr[64];
    char     RxDataMessage[256] = {0};

    int      i, j, k;
    int      MinimumMsgLength, MPT1327CmdLength;

    MPT1327CmdLength = CmdLength;
    int      DataLength = 0;

    TCanvas *pCanvas = CFG_Form_Config->Canvas;

    for(i=0 ;i<MPT1327CmdLength;i++)
      RadioCmdMsg[i].HexByte = MPT1327SerialPtr[i];
    
    //Now get description of each byte in the string.
    switch(RadioCmdMsg[0].HexByte)
    {
    case RECEIVE_STATUS :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Receive Status (5.2.2.1.2)";
         RadioCmdMsg[1].Message = "Calling Party PFIX2";
         RadioCmdMsg[2].Message = "Calling Party IDENT 2a";
         RadioCmdMsg[3].Message = "Calling Party IDENT 2b";
         
         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         RadioCmdMsg[5+j].Message = "Status 0-31";

         //------------------------------------
         //Build string for display

         //CALLER'S UNIT NUMBER
         //Callers Unit number = Ident - own IBI + 20
         //(assume caller is in the same fleet)
         MyIBI =((MyRadiosIBIa*256)+MyRadiosIBIb);
         Unit = ((RadioCmdMsg[2].HexByte*256)+RadioCmdMsg[3].HexByte)- MyIBI;
         if (MyRadiosFleetSize)
            Unit += 200;
         else
            Unit += 20;
         itoa(Unit,UnitStr,10);
         
         //Caller's Unit Number
         for(k=0;k<3;k++)      
         {
            if(UnitStr[k] == 0)
               RxDataMessage[k] = ' ';
            else   
               RxDataMessage[k] = UnitStr[k];
         }
         RxDataMessage[3] = ' ';
         RxDataMessage[4] = ' ';
         
         //TYPE OF DATA CALL
         RxDataMessage[5] = 'S';
         RxDataMessage[6] = 'T';
         RxDataMessage[7] = 'A';
         RxDataMessage[8] = 'T';
         RxDataMessage[9] = 'U';
         RxDataMessage[10] = 'S';
         RxDataMessage[11] = ' ';
         RxDataMessage[12] = ' ';
         
         //DATA
         Status = RadioCmdMsg[5+j].HexByte;
         itoa(Status,StatusStr,10);

         //Caller's Unit Number
         for(k=0;k<2;k++)
         {
            if(StatusStr[k] == 0)
               RxDataMessage[13+k] = ' ';
            else
               RxDataMessage[13+k] = StatusStr[k];
         }
         RxDataMessage[13+k] = 0;
                              
         DisplayRxSerialData(RxDataMessage);
         
         break;
         
    case ACK_POSITIVE :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Ack Positive";
         RadioCmdMsg[1].Message = "Called Party PFIX1";
         RadioCmdMsg[2].Message = "Called Party IDENT 1a";
         RadioCmdMsg[3].Message = "Called Party IDENT 1b";
         
         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         RadioCmdMsg[5+j].Message = "Cause: Call Connected";
         //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = " ";
         MF_Form_MainForm->MF_Label_CallStatus->Caption = " ";
         break;  
         
    case ACK_QUEUING :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Ack Queuing";
         RadioCmdMsg[1].Message = "Called Party PFIX1";
         RadioCmdMsg[2].Message = "Called Party IDENT 1a";
         RadioCmdMsg[3].Message = "Called Party IDENT 1b";
         
         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";
            
         if(RadioCmdMsg[5+j].HexByte == 0x02)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKQ System Busy";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Call Queued";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Call Queued";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x0A)        
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKQ Called Unit Engaged";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Called Unit Engaged";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Called Unit Engaged";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x26)                     
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Diverted";
         else if(RadioCmdMsg[5+j].HexByte == 0x22)
            RadioCmdMsg[5+j].Message = "Cause: TSC dosn't support MST";
         else   
            RadioCmdMsg[5+j].Message = "Cause: UNKNOWN";
         break;
         
    case ACK_NEGATIVE :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Ack Negative";
         RadioCmdMsg[1].Message = "Called Party PFIX1";
         RadioCmdMsg[2].Message = "Called Party IDENT 1a";
         RadioCmdMsg[3].Message = "Called Party IDENT 1b";
         
         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";
            
         if(RadioCmdMsg[5+j].HexByte == 0x08)
         {
            RadioCmdMsg[5+j].Message = "Cause: Transaction Aborted";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Called Cancelled";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Call Cancelled";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x03)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKX Invalid Call";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Invalid Call";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Invalid Call";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x0B)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKX System Overload";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "System Overload";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "System Overload";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x04)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKV Called Radio OutOfReach";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Radio Out-of-Reach";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Radio Out-of-Reach";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x0C)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKV Called Unit Engaged";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Called Unit Engaged";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Called Unit Engaged";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x06)
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Diverted";
         else if(RadioCmdMsg[5+j].HexByte == 0x16)
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Divrted to Grp";
         else if(RadioCmdMsg[5+j].HexByte == 0x0E)
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Diverted to Unknown Addr";
         else
            RadioCmdMsg[5+j].Message = "Cause: UNKNOWN";
         break;

    case RECEIVE_SST :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Receive SST (5.2.2.2.2)";
         RadioCmdMsg[1].Message = "Calling Party PFIX2";
         RadioCmdMsg[2].Message = "Calling Party IDENT 2a";
         RadioCmdMsg[3].Message = "Calling Party IDENT 2b";

         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         RadioCmdMsg[5+j].Message = "CODING NLB";
         for(k=0;k<(RadioCmdMsg[5+j].HexByte&0x0F); k++)
            RadioCmdMsg[6+j+k].Message = "Message Data";

         //------------------------------------
         //Build string for display
         
         //Get length of data string   
         DataLength = CmdLength - (6+j);

         //CALLER'S UNIT NUMBER
         //Callers Unit number = Ident - own IBI 
         //(assume caller is in the same fleet)
         MyIBI =((MyRadiosIBIa*256)+MyRadiosIBIb);
         Unit = ((RadioCmdMsg[2].HexByte*256)+RadioCmdMsg[3].HexByte)- MyIBI;
         if (MyRadiosFleetSize)
            Unit += 200;
         else
            Unit += 20;
         itoa(Unit,UnitStr,10);

         //Caller's Unit Number
         for(k=0;k<3;k++)
         {
            if(UnitStr[k] == 0)
               RxDataMessage[k] = ' ';
            else
               RxDataMessage[k] = UnitStr[k];
         }
         RxDataMessage[3] = ' ';
         RxDataMessage[4] = ' ';

         //TYPE OF DATA CALL
         RxDataMessage[5] = 'S';
         RxDataMessage[6] = 'D';
         RxDataMessage[7] = 'M';
         RxDataMessage[8] = ':';
         RxDataMessage[9] = ' ';

         //------------- CR001
         //SDM DATA is either ascii or binary

         if((RadioCmdMsg[5+j].HexByte&0xF0) == 0xb0)
         {
            //Must be an ascii SDM
            for(k=0;k<DataLength;k++)
               RxDataMessage[10+k] = (char)RadioCmdMsg[6+j+k].HexByte;
            RxDataMessage[(10+j+k)] = 0;
         }
         else if((RadioCmdMsg[5+j].HexByte&0xF0) == 0x80)
         {
            //Must be a binary SDM
            // |8n|Type|Status|JobNo[8]|GPSData[12]|

            if(RadioCmdMsg[6+j].HexByte == 0x01)
            {
                //Must be Status + GPS    (Note not the same as Status(+GPS) )
               Status = RadioCmdMsg[7+j].HexByte;
               itoa(Status,StatusStr,10);
               for(k=0;k<2;k++)
               {
                  if(StatusStr[k] == 0)
                     RxDataMessage[10+k] = ' ';
                  else
                     RxDataMessage[10+k] = StatusStr[k];
               }
               RxDataMessage[12] = ' ';

               //Prepare GPS string
               GPSStr[0] = 'T';
               GPSStr[1] = 'i';
               GPSStr[2] = 'm';
               GPSStr[3] = 'e';
               GPSStr[4] = ' ';
               GPSStr[5] = 0x30+(((RadioCmdMsg[8+j].HexByte) & 0xf0)>>4);
               GPSStr[6] = 0x30+(((RadioCmdMsg[8+j].HexByte) & 0x0f));
               GPSStr[7] = ':';
               GPSStr[8] = 0x30+(((RadioCmdMsg[9+j].HexByte) & 0xf0)>>4);
               GPSStr[9] = 0x30+(((RadioCmdMsg[9+j].HexByte) & 0x0f));
               GPSStr[10] = ':';
               GPSStr[11] = 0x30+(((RadioCmdMsg[10+j].HexByte) & 0xf0)>>4);
               GPSStr[12] = 0x30+(((RadioCmdMsg[10+j].HexByte) & 0x0f));
               GPSStr[13] = ' ';
               GPSStr[14] = 'L';
               GPSStr[15] = 'a';
               GPSStr[16] = 't';
               GPSStr[17] = ' ';
               GPSStr[18] = 0x30+(((RadioCmdMsg[11+j].HexByte) & 0xf0)>>4);
               GPSStr[19] = 0x30+(((RadioCmdMsg[11+j].HexByte) & 0x0f));
               GPSStr[20] = 0x30+(((RadioCmdMsg[12+j].HexByte) & 0xf0)>>4);
               GPSStr[21] = '.';
               GPSStr[22] = 0x30+(((RadioCmdMsg[12+j].HexByte) & 0x0f));
               GPSStr[23] = 0x30+(((RadioCmdMsg[13+j].HexByte) & 0xf0)>>4);
               GPSStr[24] = '.';
               GPSStr[25] = 0x30+(((RadioCmdMsg[13+j].HexByte) & 0x0f));
               GPSStr[26] = 0x30+(((RadioCmdMsg[14+j].HexByte) & 0xf0)>>4);
               GPSStr[27] = 0x30+(((RadioCmdMsg[14+j].HexByte) & 0x0f));
               GPSStr[28] = ' ';
               GPSStr[29] = 'L';
               GPSStr[30] = 'o';
               GPSStr[31] = 'n';
               GPSStr[32] = 'g';
               GPSStr[33] = ' ';
               GPSStr[34] = 0x30+(((RadioCmdMsg[15+j].HexByte) & 0xf0)>>4);
               GPSStr[35] = 0x30+(((RadioCmdMsg[15+j].HexByte) & 0x0f));
               GPSStr[36] = 0x30+(((RadioCmdMsg[16+j].HexByte) & 0xf0)>>4);
               GPSStr[37] = '.';
               GPSStr[38] = 0x30+(((RadioCmdMsg[16+j].HexByte) & 0x0f));
               GPSStr[39] = 0x30+(((RadioCmdMsg[17+j].HexByte) & 0xf0)>>4);
               GPSStr[40] = '.';
               GPSStr[41] = 0x30+(((RadioCmdMsg[17+j].HexByte) & 0x0f));
               GPSStr[42] = 0x30+(((RadioCmdMsg[18+j].HexByte) & 0xf0)>>4);
               GPSStr[43] = 0x30+(((RadioCmdMsg[18+j].HexByte) & 0x0f)); 
               GPSStr[44] = 0;

               for (k=0; k<45; k++)
                  RxDataMessage[k+13] = GPSStr[k];

            }
            else if(RadioCmdMsg[6+j].HexByte == 0x02)
            {
                //Must be Status + Job Number
               Status = RadioCmdMsg[7+j].HexByte;
               itoa(Status,StatusStr,10);
               for(k=0;k<2;k++)
               {
                  if(StatusStr[k] == 0)
                     RxDataMessage[10+k] = ' ';
                  else
                     RxDataMessage[10+k] = StatusStr[k];
               }
               RxDataMessage[12] = ' ';

               for(k=0;k<8;k++)
               {
                  if(RadioCmdMsg[6+j+2+k].HexByte == 0)
                     RxDataMessage[13+k] = ' ';
                  else
                     RxDataMessage[13+k] = RadioCmdMsg[6+j+2+k].HexByte;
               }
               RxDataMessage[21] = 0;

            }
            else if(RadioCmdMsg[6+j].HexByte == 0x03)
            {
                //Must be Status + Job Number + GPS
               Status = RadioCmdMsg[7+j].HexByte;
               itoa(Status,StatusStr,10);
               for(k=0;k<2;k++)
               {
                  if(StatusStr[k] == 0)
                     RxDataMessage[10+k] = ' ';
                  else
                     RxDataMessage[10+k] = StatusStr[k];
               }
               RxDataMessage[12] = ' ';

               for(k=0;k<8;k++)
               {
                  if(RadioCmdMsg[6+j+2+k].HexByte == 0)
                     RxDataMessage[13+k] = ' ';
                  else
                     RxDataMessage[13+k] = RadioCmdMsg[6+j+2+k].HexByte;
               }
               RxDataMessage[21] = ' ';
               
               //Prepare GPS string
               GPSStr[0] = 'T';
               GPSStr[1] = 'i';
               GPSStr[2] = 'm';
               GPSStr[3] = 'e';
               GPSStr[4] = ' ';
               GPSStr[5] = 0x30+(((RadioCmdMsg[8+j].HexByte) & 0xf0)>>4);
               GPSStr[6] = 0x30+(((RadioCmdMsg[8+j].HexByte) & 0x0f));
               GPSStr[7] = ':';
               GPSStr[8] = 0x30+(((RadioCmdMsg[9+j].HexByte) & 0xf0)>>4);
               GPSStr[9] = 0x30+(((RadioCmdMsg[9+j].HexByte) & 0x0f));
               GPSStr[10] = ':';
               GPSStr[11] = 0x30+(((RadioCmdMsg[10+j].HexByte) & 0xf0)>>4);
               GPSStr[12] = 0x30+(((RadioCmdMsg[10+j].HexByte) & 0x0f));
               GPSStr[13] = ' ';
               GPSStr[14] = 'L';
               GPSStr[15] = 'a';
               GPSStr[16] = 't';
               GPSStr[17] = ' ';
               GPSStr[18] = 0x30+(((RadioCmdMsg[11+j].HexByte) & 0xf0)>>4);
               GPSStr[19] = 0x30+(((RadioCmdMsg[11+j].HexByte) & 0x0f));
               GPSStr[20] = 0x30+(((RadioCmdMsg[12+j].HexByte) & 0xf0)>>4);
               GPSStr[21] = '.';
               GPSStr[22] = 0x30+(((RadioCmdMsg[12+j].HexByte) & 0x0f));
               GPSStr[23] = 0x30+(((RadioCmdMsg[13+j].HexByte) & 0xf0)>>4);
               GPSStr[24] = '.';
               GPSStr[25] = 0x30+(((RadioCmdMsg[13+j].HexByte) & 0x0f));
               GPSStr[26] = 0x30+(((RadioCmdMsg[14+j].HexByte) & 0xf0)>>4);
               GPSStr[27] = 0x30+(((RadioCmdMsg[14+j].HexByte) & 0x0f));
               GPSStr[28] = ' ';
               GPSStr[29] = 'L';
               GPSStr[30] = 'o';
               GPSStr[31] = 'n';
               GPSStr[32] = 'g';
               GPSStr[33] = ' ';
               GPSStr[34] = 0x30+(((RadioCmdMsg[15+j].HexByte) & 0xf0)>>4);
               GPSStr[35] = 0x30+(((RadioCmdMsg[15+j].HexByte) & 0x0f));
               GPSStr[36] = 0x30+(((RadioCmdMsg[16+j].HexByte) & 0xf0)>>4);
               GPSStr[37] = '.';
               GPSStr[38] = 0x30+(((RadioCmdMsg[16+j].HexByte) & 0x0f));
               GPSStr[39] = 0x30+(((RadioCmdMsg[17+j].HexByte) & 0xf0)>>4);
               GPSStr[40] = '.';
               GPSStr[41] = 0x30+(((RadioCmdMsg[17+j].HexByte) & 0x0f));
               GPSStr[42] = 0x30+(((RadioCmdMsg[18+j].HexByte) & 0xf0)>>4);
               GPSStr[43] = 0x30+(((RadioCmdMsg[18+j].HexByte) & 0x0f)); 
               GPSStr[44] = 0;

               for (k=0; k<45; k++)
                  RxDataMessage[k+22] = GPSStr[k];
   
            }
            else
            {
                //binary data
                for(k=0;k<DataLength;k++)
                   RxDataMessage[10+k] = (char)RadioCmdMsg[6+j+k].HexByte;
                RxDataMessage[(10+j+k)] = 0;
            }
         }
         else
         {
             //Unknown
         }

         //------------- CR001

         DisplayRxSerialData(RxDataMessage);

         break;
         
    case RECEIVE_MST :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Receive MST (5.2.2.3.2)";
         RadioCmdMsg[1].Message = "Calling Party PFIX2";
         RadioCmdMsg[2].Message = "Calling Party IDENT 2a";
         RadioCmdMsg[3].Message = "Calling Party IDENT 2b";

         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         RadioCmdMsg[5+j].Message = "CODING NLB";
         for(k=0;k<(RadioCmdMsg[5+j].HexByte&0x0F); k++)
            RadioCmdMsg[6+j+k].Message = "Message Data";

         //------------------------------------
         //Build string for display
         
         //Get length of data string
         DataLength = CmdLength - (6+j);   
         
         //CALLER'S UNIT NUMBER
         //Callers Unit number = Ident - own IBI 
         //(assume caller is in the same fleet)
         MyIBI =((MyRadiosIBIa*256)+MyRadiosIBIb);
         Unit = ((RadioCmdMsg[2].HexByte*256)+RadioCmdMsg[3].HexByte)- MyIBI;
         if (MyRadiosFleetSize)
            Unit += 200;
         else
            Unit += 20;
         itoa(Unit,UnitStr,10);

         //Caller's Unit Number
         for(k=0;k<3;k++)
         {
            if(UnitStr[k] == 0)
               RxDataMessage[k] = ' ';
            else   
               RxDataMessage[k] = UnitStr[k];
         }   
         RxDataMessage[3] = ' ';
         RxDataMessage[4] = ' ';
         
         //TYPE OF DATA CALL
         RxDataMessage[5] = 'E';
         RxDataMessage[6] = 'D';
         RxDataMessage[7] = 'M';
         RxDataMessage[8] = ' ';
         RxDataMessage[9] = ' ';
         RxDataMessage[10] = ' ';
         RxDataMessage[11] = ' ';
         RxDataMessage[12] = ' ';
         
         //DATA
         for(k=0;k<DataLength;k++)
            RxDataMessage[13+k] = (char)RadioCmdMsg[6+j+k].HexByte;
         RxDataMessage[(13+j+k)] = 0;
                              
         DisplayRxSerialData(RxDataMessage);
            
         break;
         
    case SETUP_PROGRESS_POS :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Setup Progress Positive (5.2.2.4.3)";
         RadioCmdMsg[1].Message = "Called Party PFIX1";
         RadioCmdMsg[2].Message = "Called Party IDENT 1a";
         RadioCmdMsg[3].Message = "Called Party IDENT 1b";

         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";
            
         RadioCmdMsg[5+j].Message = "Cause: Call Connected";
         //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = " ";
         MF_Form_MainForm->MF_Label_CallStatus->Caption = " ";
         
         CFG_Form_Config->CFG_Btn_SetupModem->Caption = "CLEAR DOWN"; //V1.05
         
         break;
         
    case SETUP_PROGRESS_QUE :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Setup Progress Queuing (5.2.2.4.3)";
         RadioCmdMsg[1].Message = "Called Party PFIX1";
         RadioCmdMsg[2].Message = "Called Party IDENT 1a";
         RadioCmdMsg[3].Message = "Called Party IDENT 1b";
         
         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";
            
         if(RadioCmdMsg[5+j].HexByte == 0x01)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKI Called Unit Alerting";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Ringing - Please Wait";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Ringing - Please Wait";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x02)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKQ System Busy";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Call Queued";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Call Queued";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x0A)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKQ Called Unit Engaged";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Called Unit Engaged";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Called Unit Engaged";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x05)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKE Emergency Call Proceeding";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x26)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Diverted";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x36)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Diverted to Grp";
         }
         else
            RadioCmdMsg[5+j].Message = "Cause: UNKNOWN";


         break;

    case SETUP_PROGRESS_NEG :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Setup Progress Negative (5.2.2.4.3)";
         RadioCmdMsg[1].Message = "Called Party PFIX1";
         RadioCmdMsg[2].Message = "Called Party IDENT 1a";
         RadioCmdMsg[3].Message = "Called Party IDENT 1b";

         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         if(RadioCmdMsg[5+j].HexByte == 0x08)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACK Call Setup Aborted";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Call Cancelled";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Call Cancelled";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x03)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKX Invalid Call";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Invalid Call";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Invalid Call";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x0B)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKX System Overload";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "System Overload";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "System Overload";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x04)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKV Called Radio OutOfReach";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Radio out of reach";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Radio out of reach";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x0C)
         {
            RadioCmdMsg[5+j].Message = "Cause: ACKV Called Unit Engaged";
            //CFG_Form_Config->CFG_Lbl_CallStatus->Caption = "Called Radio Engaged";
            MF_Form_MainForm->MF_Label_CallStatus->Caption = "Called Radio Engaged";
         }
         else if(RadioCmdMsg[5+j].HexByte == 0x06)
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Diverted";
         else if(RadioCmdMsg[5+j].HexByte == 0x16)
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Divrted to Grp";
         else if(RadioCmdMsg[5+j].HexByte == 0x0E)
            RadioCmdMsg[5+j].Message = "Cause: ACKT Called Unit Diverted to Unknown Addr";
         else if(RadioCmdMsg[5+j].HexByte == 0x07)
            RadioCmdMsg[5+j].Message = "Cause: ACKB Called accepted call for callback";
         else
            RadioCmdMsg[5+j].Message = "Cause: UNKNOWN";

         CFG_Form_Config->CFG_Btn_SetupModem->Caption = "Setup NPD Call"; //V1.05
            
         break;

    case INCOMING_CALL :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Incomming Call (5.2.2.4.4)";
         RadioCmdMsg[1].Message = "Calling Party PFIX2";
         RadioCmdMsg[2].Message = "Calling Party IDENT 2a";
         RadioCmdMsg[3].Message = "Calling Party IDENT 2b";

         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         RadioCmdMsg[5+j].Message = "Call Details";      //Add BIT definitions
                                                         //see if \n can be added
         break;

    case INCOMING_EMERG_CALL :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Incomming Emergency Call (5.2.2.4.5)";
         RadioCmdMsg[1].Message = "Calling Party PFIX2";
         RadioCmdMsg[2].Message = "Calling Party IDENT 2a";
         RadioCmdMsg[3].Message = "Calling Party IDENT 2b";

         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         RadioCmdMsg[5+j].Message = "Call Details";      //Add BIT definitions
                                                         //see if \n can be added
         break;

    case RECEIVE_PROGRESS_POS :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Receive Progress Positive (5.2.2.4.6)";
         RadioCmdMsg[1].Message = "Calling Party PFIX2";
         RadioCmdMsg[2].Message = "Calling Party IDENT 2a";
         RadioCmdMsg[3].Message = "Calling Party IDENT 2b";

         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         if(RadioCmdMsg[5+j].HexByte == 0x00)
         {
            RadioCmdMsg[5+j].Message = "Cause: GTC Call Connected";
            CFG_Form_Config->CFG_Btn_SetupModem->Caption = "CLEAR DOWN"; //V1.05
         }
         else
            RadioCmdMsg[5+j].Message = "Cause: UNKNOWN";

         break;
         
    case RECEIVE_PROGRESS_QUE :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Receive Progress Queuing (5.2.2.4.6)";
         RadioCmdMsg[1].Message = "Calling Party PFIX2";
         RadioCmdMsg[2].Message = "Calling Party IDENT 2a";
         RadioCmdMsg[3].Message = "Calling Party IDENT 2b";

         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";
            
         if(RadioCmdMsg[5+j].HexByte == 0x01)
            RadioCmdMsg[5+j].Message = "Cause: MAINT Transmission Disabled";
         else if(RadioCmdMsg[5+j].HexByte == 0x02)
            RadioCmdMsg[5+j].Message = "Cause: Note 1: System Busy";
         else if(RadioCmdMsg[5+j].HexByte == 0x0A)
            RadioCmdMsg[5+j].Message = "Cause: Local Clear Down Timer Warning";
         else   
            RadioCmdMsg[5+j].Message = "Cause: UNKNOWN";

         break;
         
    case RECEIVE_PROGRESS_NEG :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Receive Progress Negative (5.2.2.4.6)";
         RadioCmdMsg[1].Message = "Calling Party PFIX2";
         RadioCmdMsg[2].Message = "Calling Party IDENT 2a";
         RadioCmdMsg[3].Message = "Calling Party IDENT 2b";
         
         RadioCmdMsg[4].Message = "ADESC LENGTH";
         for(j=0;j<(RadioCmdMsg[4].HexByte&0x0F); j++)
            RadioCmdMsg[5+j].Message = "  ";

         if(RadioCmdMsg[5+j].HexByte == 0x03)
            RadioCmdMsg[5+j].Message = "Cause: AHYX Call Setup Rejected";
         else if(RadioCmdMsg[5+j].HexByte == 0x07)
            RadioCmdMsg[5+j].Message = "Cause: Local Radio accepted call for callback";
         else   
            RadioCmdMsg[5+j].Message = "Cause: UNKNOWN";
            
         CFG_Form_Config->CFG_Btn_SetupModem->Caption = "Setup NPD Call"; //V1.05
         break;
         
    case RECEIVE_MODEM_DATA :
         MinimumMsgLength = 2;
         RadioCmdMsg[0].Message = "Receive Modem Data (5.2.2.5.2)";
         RadioCmdMsg[1].Message = "User Data (free format)";
         for(j=0;j<MPT1327CmdLength; j++)
            RadioCmdMsg[2+j].Message = "  ";


         //TYPE OF DATA CALL
         RxDataMessage[0] = ' ';
         RxDataMessage[1] = ' ';
         RxDataMessage[2] = ' ';
         RxDataMessage[3] = ' ';
         RxDataMessage[4] = ' ';
         RxDataMessage[5] = 'N';
         RxDataMessage[6] = 'P';
         RxDataMessage[7] = 'D';
         RxDataMessage[8] = ' ';
         RxDataMessage[9] = ' ';
         RxDataMessage[10] = ' ';
         RxDataMessage[11] = ' ';
         RxDataMessage[12] = ' ';
         
         //DATA
         //Get length of data string
         DataLength = CmdLength - (2);

         for(k=0;k<DataLength;k++)
            RxDataMessage[13+k] = (char)RadioCmdMsg[2+k].HexByte;
         RxDataMessage[(13+k)] = 0;

         DisplayRxSerialData(RxDataMessage);

         break;

    case CONNECTION_CLR_NORM :
         MinimumMsgLength = 5;
         RadioCmdMsg[0].Message = "Cleared (Normal) (5.2.2.6.2)";
         RadioCmdMsg[1].Message = "Calling/Called PFIX1";
         RadioCmdMsg[2].Message = "Calling/Called IDENT 1a";
         RadioCmdMsg[3].Message = "Calling/Called IDENT 1b";
         if(RadioCmdMsg[4].HexByte == 0x0B)
            RadioCmdMsg[4].Message = "Cause: Voice/Modem call disconnected";
         else   
            RadioCmdMsg[4].Message = "Cause: UNKNOWN";
            
         CFG_Form_Config->CFG_Btn_SetupModem->Caption = "Setup NPD Call"; //V1.05
         break;
         
    case CONNECTION_CLR_ABNORM :
         MinimumMsgLength = 5;
         RadioCmdMsg[0].Message = "Cleared (Abnormal) (5.2.2.6.2)";
         RadioCmdMsg[1].Message = "Calling/Called PFIX1";
         RadioCmdMsg[2].Message = "Calling/Called IDENT 1a";
         RadioCmdMsg[3].Message = "Calling/Called IDENT 1b";
         if(RadioCmdMsg[4].HexByte == 0x00)
            RadioCmdMsg[4].Message = "Cause: CLEAR-All msg transactions cleared";
         else if(RadioCmdMsg[4].HexByte == 0x01)
            RadioCmdMsg[4].Message = "Cause: Local-Radio generated clear";
         else if(RadioCmdMsg[4].HexByte == 0x04)
            RadioCmdMsg[4].Message = "Cause: Local-Service not available";
         else if(RadioCmdMsg[4].HexByte == 0x05)
            RadioCmdMsg[4].Message = "Cause: Local-Transmission or msg too long (CallFail)";
         else if(RadioCmdMsg[4].HexByte == 0x06)
            RadioCmdMsg[4].Message = "Cause: Local-Message coding not possible";
         else if(RadioCmdMsg[4].HexByte == 0x0E)
            RadioCmdMsg[4].Message = "Cause: MAINT-Voice or modem disconnected";
         else
            RadioCmdMsg[4].Message = "Cause: UNKNOWN";
                                             
         CFG_Form_Config->CFG_Btn_SetupModem->Caption = "Setup NPD Call"; //V1.05
         break;
         
    case DIVERSION_ACK_POS :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Diversion Ack Positive (5.2.2.7.3)";
         RadioCmdMsg[1].Message = "Divert Target PFIX1";
         RadioCmdMsg[2].Message = "Divert Target IDENT 1a";
         RadioCmdMsg[3].Message = "Divert Target IDENT 1b";
         RadioCmdMsg[4].Message = "ADESC LENGTH=0";
         
         if(RadioCmdMsg[5].HexByte == 0x00)
            RadioCmdMsg[5].Message = "Cause: ACK Call divert or cancel accepted";
         else   
            RadioCmdMsg[5].Message = "Cause: UNKNOWN";

         break;

    case DIVERSION_ACK_NEG :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Diversion Ack Negative (5.2.2.7.3)";
         RadioCmdMsg[1].Message = "Divert Target PFIX1";
         RadioCmdMsg[2].Message = "Divert Target IDENT 1a";
         RadioCmdMsg[3].Message = "Divert Target IDENT 1b";
         RadioCmdMsg[4].Message = "ADESC LENGTH=0";
         
         if(RadioCmdMsg[5].HexByte == 0x03)
            RadioCmdMsg[5].Message = "Cause: ACKX Invalid Diversion";
         else if(RadioCmdMsg[5].HexByte == 0x0B)
            RadioCmdMsg[5].Message = "Cause: ACKX System Overload";
         else if(RadioCmdMsg[5].HexByte == 0x04)
            RadioCmdMsg[5].Message = "Cause: ACKV Transaction Abandoned";
         else   
            RadioCmdMsg[5].Message = "Cause: UNKNOWN";
                                                                   
         break;

    case RADIO_PERSONALITY :
         MinimumMsgLength = 12;
         RadioCmdMsg[0].Message = "Radio Personality (5.2.2.8.2)";
         RadioCmdMsg[1].Message = "Radio Unit PFIX1";
         RadioCmdMsg[2].Message = "Radio Unit IDENT 1a";
         RadioCmdMsg[3].Message = "Radio Unit IDENT 1b";
         RadioCmdMsg[4].Message = "Manufacturers Code";
         RadioCmdMsg[5].Message = "Model:8-5, Serial:2-1";
         RadioCmdMsg[6].Message = "Serial-B";
         RadioCmdMsg[7].Message = "Serial-C";
         RadioCmdMsg[8].Message = "Supported Facilities A";         //Add BIT definitions
         RadioCmdMsg[9].Message = "Supported Facilities B = 0";
         RadioCmdMsg[10].Message = "Supported Facilities C = 0";
         RadioCmdMsg[11].Message = "Supported Codings ";
         
         break;
         
    case NUMBERING_INFO :
         MinimumMsgLength = 15;
         RadioCmdMsg[0].Message = "Numbering Information (5.2.2.8.3)";
         RadioCmdMsg[1].Message = "Radio Unit PFIX1";
         RadioCmdMsg[2].Message = "Radio Unit IDENT 1a";
         RadioCmdMsg[3].Message = "Radio Unit IDENT 1b";
         RadioCmdMsg[4].Message = "L + Individual Base A ";
         RadioCmdMsg[5].Message = "Individual Base B ";
         RadioCmdMsg[6].Message = "Highest Ident A ";
         RadioCmdMsg[7].Message = "Highest Ident B ";
         RadioCmdMsg[8].Message = "L + Group Base A ";
         RadioCmdMsg[9].Message = "Group Base B ";
         RadioCmdMsg[10].Message = "Highest Group A ";
         RadioCmdMsg[11].Message = "Highest Group B ";
         RadioCmdMsg[12].Message = "Group PFIX1 ";
         RadioCmdMsg[13].Message = "Group IDENT 1a ";
         RadioCmdMsg[14].Message = "Group IDENT 1b ";
         for(j=0;j<MPT1327CmdLength; j++)
            RadioCmdMsg[15+j].Message = "  ";

         //Set global variables describing current radio
         MyRadiosPrefix     = RadioCmdMsg[1].HexByte;
         MyRadiosIBIa       = RadioCmdMsg[4].HexByte & 0x7f;
         MyRadiosIBIb       = RadioCmdMsg[5].HexByte;
         MyRadiosFleetSize  = (RadioCmdMsg[4].HexByte & 0x80)>>7; //V1.04 Bug
         MyRadiosHighestId  = (RadioCmdMsg[6].HexByte<<8)+RadioCmdMsg[7].HexByte;
         MyRadiosGBIa       = RadioCmdMsg[8].HexByte & 0x7f;
         MyRadiosGBIb       = RadioCmdMsg[9].HexByte;
         MyRadiosGrpFleetSize = (RadioCmdMsg[8].HexByte & 0x80)>>7; //V1.04 Bug
         MyRadiosHighestGrp = (RadioCmdMsg[10].HexByte<<8)+RadioCmdMsg[11].HexByte;

         //Tell user we have the info
         DisplayLED(LED2,LED_ON,LED_GREEN);
         //pCanvas->Brush->Color = clGreen;
         //MF_Form_MainForm->MF_Shape_LED2->Brush->Color = clLime;

         break;

    case OPERATING_CONDITION :
         MinimumMsgLength = 4;
         RadioCmdMsg[0].Message = "Operating Condition (5.2.2.8.5)";

         if((RadioCmdMsg[1].HexByte&0x04) == 0x00)
         {
            RadioCmdMsg[1].Message = "Conditions: Out of Service";
            DisplayLED(LED3,LED_OFF,LED_BLANK);
         }
         else if((RadioCmdMsg[1].HexByte&0x04) == 0x04)
         {
            RadioCmdMsg[1].Message = "Conditions: In Service";
            DisplayLED(LED3,LED_ON,LED_GREEN);
         }
         else
            RadioCmdMsg[1].Message = "Conditions";

         //Set BUSY LED
         if((RadioCmdMsg[1].HexByte&0x02) == 0x00)
         {
            DisplayLED(LED4,LED_OFF,LED_BLANK);
         }
         else if((RadioCmdMsg[1].HexByte&0x02) == 0x02)
         {
            DisplayLED(LED4,LED_ON,LED_GREEN);
         }

         //Set TRANSMIT LED
         if((RadioCmdMsg[1].HexByte&0x01) == 0x00)
         {
            DisplayLED(LED5,LED_OFF,LED_BLANK);
         }
         else if((RadioCmdMsg[1].HexByte&0x01) == 0x01)
         {
            DisplayLED(LED5,LED_ON,LED_RED);
         }

         RadioCmdMsg[2].Message = "Field Strength";

         if(RadioCmdMsg[3].HexByte == 0x00)
            RadioCmdMsg[3].Message = "MCDT: Reserved";
         else if(RadioCmdMsg[3].HexByte == 0x01)
            RadioCmdMsg[3].Message = "MCDT: Call duration min (5min)";
         else if(RadioCmdMsg[3].HexByte == 0x09)
            RadioCmdMsg[3].Message = "MCDT: Call duration min (13min)";
         else if(RadioCmdMsg[3].HexByte == 0x0A)
            RadioCmdMsg[3].Message = "MCDT: Call duration sec (10sec)";
         else if(RadioCmdMsg[3].HexByte == 0xFE)
            RadioCmdMsg[3].Message = "MCDT: Call duration sec (254sec)";
         else if(RadioCmdMsg[3].HexByte == 0xFF)
            RadioCmdMsg[3].Message = "MCDT: Timer indefinite / Not supported";
         else
            RadioCmdMsg[3].Message = "MCDT: UNKNOWN";

         break;

    case NETWORK_INFO :
         MinimumMsgLength = 5;
         RadioCmdMsg[0].Message = "Network Information (5.2.2.8.6)";
         RadioCmdMsg[1].Message = "Radio Channel Number :2";
         RadioCmdMsg[2].Message = "Radio Channel Number :8";
         RadioCmdMsg[3].Message = "System Identity :2";
         RadioCmdMsg[4].Message = "System Identity :8";
    
         break;

    case RADIO_SETTINGS :
         MinimumMsgLength = 4;
         RadioCmdMsg[0].Message = "Radio Settings (5.2.2.8.8)";
         RadioCmdMsg[1].Message = "Radio Settings A";
         RadioCmdMsg[2].Message = "Radio Settings B = 0";
         RadioCmdMsg[3].Message = "Radio Settings C = 0";

         break;
         
    case PROTOCOL_INFO :
         MinimumMsgLength = 2;
         RadioCmdMsg[0].Message = "Protocol Information (5.2.2.8.9)";
         if(RadioCmdMsg[1].HexByte == 0x01)
            RadioCmdMsg[1].Message = "Reason: Unrecognised Message ";
         else if(RadioCmdMsg[1].HexByte == 0x02)
            RadioCmdMsg[1].Message = "Reason: Facility or Addressing not supported";
         else if(RadioCmdMsg[1].HexByte == 0x03)
            RadioCmdMsg[1].Message = "Reason: Protocol state mismatch detected";
         else if(RadioCmdMsg[1].HexByte == 0x04)
            RadioCmdMsg[1].Message = "Reason: Message coding not supported";
         else if((RadioCmdMsg[1].HexByte&0x10) == 0x10)
            RadioCmdMsg[1].Message = "Reason: Spare for Cusomisation ";
         else
            RadioCmdMsg[1].Message = "Reason: UNKNOWN";
                                                       
         break;

    case RADIO_TEST_RESULTS :
         MinimumMsgLength = 2;
         RadioCmdMsg[0].Message = "Radio Test Results (5.2.2.8.12)";
         RadioCmdMsg[1].Message = "Test Results";
         for(j=0;j<MPT1327CmdLength; j++)
            RadioCmdMsg[2+j].Message = "  ";

         break;
         
    default:
         MinimumMsgLength = 1;
         RadioCmdMsg[0].Message = "Unkown Command";
         for(j=0;j<MPT1327CmdLength; j++)
            RadioCmdMsg[1+j].Message = "  ";

        break;
    }

    //Clear current string grid
    for(i=0; i<CFG_Form_Config->CFG_StrGrid_CmdDefinition->RowCount; i++)
    {
        CFG_Form_Config->CFG_StrGrid_CmdDefinition->Cells[0][i] = " ";
        CFG_Form_Config->CFG_StrGrid_CmdDefinition->Cells[1][i] = " ";
    }

    //copy bytes and difinitions to list
    char HexByteStr[3] = " ";

    for(i=0; i<MPT1327CmdLength; i++)
    {
         highNibble = (RadioCmdMsg[i].HexByte & 0xF0) >> 4;
         lowNibble  = RadioCmdMsg[i].HexByte & 0x0F;

         if(highNibble <= 0x09)
            HexByteStr[0] = 48 + highNibble;
         else
            HexByteStr[0] = 65 + highNibble - 10;

         if(lowNibble <= 0x09)
            HexByteStr[1] = 48 + lowNibble;
         else
            HexByteStr[1] = 65 + lowNibble - 10;
         HexByteStr[2] = 0;

        CFG_Form_Config->CFG_StrGrid_CmdDefinition->Cells[0][i] = HexByteStr;
        CFG_Form_Config->CFG_StrGrid_CmdDefinition->Cells[1][i] = RadioCmdMsg[i].Message;
    }
    for(i=MPT1327CmdLength; i<46; i++)
    {
        CFG_Form_Config->CFG_StrGrid_CmdDefinition->Cells[0][i] = " ";
        CFG_Form_Config->CFG_StrGrid_CmdDefinition->Cells[1][i] = " ";
    }
    CFG_Form_Config->CFG_StrGrid_CmdDefinition->Col = 0;
    CFG_Form_Config->CFG_StrGrid_CmdDefinition->Row = 0;

};
//---------------------------------------------------------------------------
//
//  Get PMR Command definition.
//
//
//
//---------------------------------------------------------------------------
void  GetPmrCmdDefinition(unsigned char *MPT1327SerialPtr, int CmdLength)
{

    char          buffer[1024];
    unsigned char highNibble;
    unsigned char lowNibble;
    int           PacketLength = 0;
    int           Status;
    int           Indicator;

    int      i, j, k;
    int      MinimumMsgLength, MPT1327CmdLength;
    char     RxDataMessage[256] = {0};
    char     StatusStr[10];
    char     JobStr[10];
    char     GPSStr[64];

    TCanvas *pCanvas = CFG_Form_Config->Canvas;
    
    MPT1327CmdLength = CmdLength;
    CmdLength = CmdLength -1;

    //Clear arrays
    for(i=0 ;i<40;i++)
    {
       RadioCmdMsg[i].HexByte = 0;
       RadioCmdMsg[1].Message = "";
    }

    for(i=0 ;i<MPT1327CmdLength;i++)
       RadioCmdMsg[i].HexByte = MPT1327SerialPtr[i+1];

    //Now get description of each byte in the string.
    switch(RadioCmdMsg[0].HexByte)
    {
    case STATE_REPORT :
         MinimumMsgLength = 3;
         RadioCmdMsg[0].Message = "Status Report";

         if(RadioCmdMsg[1].HexByte == 0)
         {
            RadioCmdMsg[1].Message = "Response Type - Signal Data";
            RadioCmdMsg[2].Message = "Flags: - - - - - S DW G";
            RadioCmdMsg[3].Message = "Flags: V SC SD Tx - Sm Ct Cd";
            RadioCmdMsg[4].Message = "RSSI";
            RadioCmdMsg[5].Message = "Channel Number. (High Byte)";
            RadioCmdMsg[5].Message = "Channel Number. (Low Byte)";

            //Update status LEDs
            Indicator = RadioCmdMsg[3].HexByte;
            if(Indicator&0x01)
               DisplayLED(LED3,LED_ON,LED_GREEN);
            else
               DisplayLED(LED3,LED_OFF,LED_BLANK);
            if(Indicator&0x10)
               DisplayLED(LED5,LED_ON,LED_GREEN);
            else
               DisplayLED(LED5,LED_OFF,LED_BLANK);
            if(Indicator&0x80)
               DisplayLED(LED4,LED_ON,LED_GREEN);
            else
               DisplayLED(LED4,LED_OFF,LED_BLANK);

         }
         else if(RadioCmdMsg[1].HexByte == 1)
         {
            RadioCmdMsg[1].Message = "Response Type - Software Version";
            RadioCmdMsg[2].Message = "Version. Unit in BCD";
            RadioCmdMsg[3].Message = "Version. Decimal in BCD";
         }
         else if(RadioCmdMsg[1].HexByte == 2)
         {
            RadioCmdMsg[1].Message = "Response Type - GPS Data";
            RadioCmdMsg[2].Message = "Data format not yet defined.";
         }
         else
            RadioCmdMsg[1].Message = "Unknown Type";

         //Tell user we have the info
         DisplayLED(LED2,LED_ON,LED_GREEN);
         //pCanvas->Brush->Color = clGreen;
         //MF_Form_MainForm->MF_Shape_LED2->Brush->Color = clLime;
         break;

    case DATA_SEND_ACK :
         MinimumMsgLength = 2;
         RadioCmdMsg[0].Message = "Data Send Acknowledge";
         RadioCmdMsg[1].Message = "Retries";
         break;

    case RX_PMR_DATA :
         MinimumMsgLength = 6;
         RadioCmdMsg[0].Message = "Data Message Received";
         RadioCmdMsg[1].Message = "Source Radio's Individual Address (High Byte)";
         RadioCmdMsg[2].Message = "Source Radio's Individual Address (Low Byte)";

         if(RadioCmdMsg[3].HexByte == 0)
            RadioCmdMsg[3].Message = "Msg Format: ASCII Data - Alert: RingRing";
         else if(RadioCmdMsg[3].HexByte == 1)
            RadioCmdMsg[3].Message = "Msg Format: ASCII Data - Alert: Bip";
         else if(RadioCmdMsg[3].HexByte == 2)
            RadioCmdMsg[3].Message = "Msg Format: ASCII Data - Alert: BipBip";
         else if(RadioCmdMsg[3].HexByte == 3)
            RadioCmdMsg[3].Message = "Msg Format: ASCII Data - Alert: Beep";
         else if(RadioCmdMsg[3].HexByte == 4)
            RadioCmdMsg[3].Message = "Msg Format: ASCII Data - Alert: Urgent";
         else if(RadioCmdMsg[3].HexByte == 5)
            RadioCmdMsg[3].Message = "Msg Format: ASCII Data - Alert: None";
         else if(RadioCmdMsg[3].HexByte == 6)
            RadioCmdMsg[3].Message = "Msg Format: ASCII Data - Alert: None";
         else if(RadioCmdMsg[3].HexByte == 7)
            RadioCmdMsg[3].Message = "Msg Format: ASCII Data - Alert: None";
         else if(RadioCmdMsg[3].HexByte == 8)
            RadioCmdMsg[3].Message = "Msg Format: GPS Poll Request";
         else if(RadioCmdMsg[3].HexByte == 9)
            RadioCmdMsg[3].Message = "Msg Format: Channel Change Request";
         else if(RadioCmdMsg[3].HexByte == 10)
            RadioCmdMsg[3].Message = "Msg Format: Data = Status Only";
         else if(RadioCmdMsg[3].HexByte == 11)
            RadioCmdMsg[3].Message = "Msg Format: Data = Status + GPS location";
         else if(RadioCmdMsg[3].HexByte == 12)
            RadioCmdMsg[3].Message = "Msg Format: Data = Status + Job number";
         else if(RadioCmdMsg[3].HexByte == 13)
            RadioCmdMsg[3].Message = "Msg Format: Data = Status + Job + GPS";
         else
            RadioCmdMsg[3].Message = "Message Format: Unknown";

         for(j=0;j<CmdLength; j++)
            RadioCmdMsg[4+j].Message = "  ";

         //-------------------------------------------
         //Display Normal DATA messages
         if((RadioCmdMsg[3].HexByte >= 0) && (RadioCmdMsg[3].HexByte <= 7))
         {
            for(k=0; k<CmdLength; k++)
            {
               RxDataMessage[k] = (char)RadioCmdMsg[2+k].HexByte;
            }
            RxDataMessage[k] = 0;
            DisplayPmrRxSerialData(RxDataMessage);
         }

         //Prepare  JOB, STATUS, GPS strings
         if( (RadioCmdMsg[3].HexByte == 10)
           ||(RadioCmdMsg[3].HexByte == 11)
           ||(RadioCmdMsg[3].HexByte == 12)
           ||(RadioCmdMsg[3].HexByte == 13) )
         {
            //Prepare STATUS Number string
            Status = RadioCmdMsg[4].HexByte;
            itoa(Status,StatusStr,10);
         }
         if( (RadioCmdMsg[3].HexByte == 12)
           ||(RadioCmdMsg[3].HexByte == 13) )
         {
            //Prepare JOB Number strings
            for(k=0;k<8;k++)
              JobStr[k] = RadioCmdMsg[5+k].HexByte;
         }
         if( RadioCmdMsg[3].HexByte == 13)
         {
            //Prepare GPS string
            GPSStr[0] = 'T';
            GPSStr[1] = 'i';
            GPSStr[2] = 'm';
            GPSStr[3] = 'e';
            GPSStr[4] = ' ';
            GPSStr[5] = 0x30+(((RadioCmdMsg[13].HexByte) & 0xf0)>>4);
            GPSStr[6] = 0x30+(((RadioCmdMsg[13].HexByte) & 0x0f));
            GPSStr[7] = ':';
            GPSStr[8] = 0x30+(((RadioCmdMsg[14].HexByte) & 0xf0)>>4);
            GPSStr[9] = 0x30+(((RadioCmdMsg[14].HexByte) & 0x0f));
            GPSStr[10] = ':';
            GPSStr[11] = 0x30+(((RadioCmdMsg[15].HexByte) & 0xf0)>>4);
            GPSStr[12] = 0x30+(((RadioCmdMsg[15].HexByte) & 0x0f));
            GPSStr[13] = ' ';
            GPSStr[14] = 'L';
            GPSStr[15] = 'a';
            GPSStr[16] = 't';
            GPSStr[17] = ' ';
            GPSStr[18] = 0x30+(((RadioCmdMsg[16].HexByte) & 0xf0)>>4);
            GPSStr[19] = 0x30+(((RadioCmdMsg[16].HexByte) & 0x0f));
            GPSStr[20] = 0x30+(((RadioCmdMsg[17].HexByte) & 0xf0)>>4);
            GPSStr[21] = '.';
            GPSStr[22] = 0x30+(((RadioCmdMsg[17].HexByte) & 0x0f));
            GPSStr[23] = 0x30+(((RadioCmdMsg[18].HexByte) & 0xf0)>>4);
            GPSStr[24] = '.';
            GPSStr[25] = 0x30+(((RadioCmdMsg[18].HexByte) & 0x0f));
            GPSStr[26] = 0x30+(((RadioCmdMsg[19].HexByte) & 0xf0)>>4);
            GPSStr[27] = 0x30+(((RadioCmdMsg[19].HexByte) & 0x0f));
            GPSStr[28] = ' ';
            GPSStr[29] = 'L';
            GPSStr[30] = 'o';
            GPSStr[31] = 'n';
            GPSStr[32] = 'g';
            GPSStr[33] = ' ';
            GPSStr[34] = 0x30+(((RadioCmdMsg[20].HexByte) & 0xf0)>>4);
            GPSStr[35] = 0x30+(((RadioCmdMsg[20].HexByte) & 0x0f));
            GPSStr[36] = 0x30+(((RadioCmdMsg[21].HexByte) & 0xf0)>>4);
            GPSStr[37] = '.';
            GPSStr[38] = 0x30+(((RadioCmdMsg[21].HexByte) & 0x0f));
            GPSStr[39] = 0x30+(((RadioCmdMsg[22].HexByte) & 0xf0)>>4);
            GPSStr[40] = '.';
            GPSStr[41] = 0x30+(((RadioCmdMsg[22].HexByte) & 0x0f));
            GPSStr[42] = 0x30+(((RadioCmdMsg[23].HexByte) & 0xf0)>>4);
            GPSStr[43] = 0x30+(((RadioCmdMsg[23].HexByte) & 0x0f));
            GPSStr[44] = 0;
         }
         else if( RadioCmdMsg[3].HexByte == 11)
         {   
            //Prepare GPS string
            GPSStr[0] = 'T';
            GPSStr[1] = 'i';
            GPSStr[2] = 'm';
            GPSStr[3] = 'e';
            GPSStr[4] = ' ';
            GPSStr[5] = 0x30+(((RadioCmdMsg[5].HexByte) & 0xf0)>>4);
            GPSStr[6] = 0x30+(((RadioCmdMsg[5].HexByte) & 0x0f));
            GPSStr[7] = ':';
            GPSStr[8] = 0x30+(((RadioCmdMsg[6].HexByte) & 0xf0)>>4);
            GPSStr[9] = 0x30+(((RadioCmdMsg[6].HexByte) & 0x0f));
            GPSStr[10] = ':';
            GPSStr[11] = 0x30+(((RadioCmdMsg[7].HexByte) & 0xf0)>>4);
            GPSStr[12] = 0x30+(((RadioCmdMsg[7].HexByte) & 0x0f));
            GPSStr[13] = ' ';
            GPSStr[14] = 'L';
            GPSStr[15] = 'a';
            GPSStr[16] = 't';
            GPSStr[17] = ' ';
            GPSStr[18] = 0x30+(((RadioCmdMsg[8].HexByte) & 0xf0)>>4);
            GPSStr[19] = 0x30+(((RadioCmdMsg[8].HexByte) & 0x0f));
            GPSStr[20] = 0x30+(((RadioCmdMsg[9].HexByte) & 0xf0)>>4);
            GPSStr[21] = '.';
            GPSStr[22] = 0x30+(((RadioCmdMsg[9].HexByte) & 0x0f));
            GPSStr[23] = 0x30+(((RadioCmdMsg[10].HexByte) & 0xf0)>>4);
            GPSStr[24] = '.';
            GPSStr[25] = 0x30+(((RadioCmdMsg[10].HexByte) & 0x0f));
            GPSStr[26] = 0x30+(((RadioCmdMsg[11].HexByte) & 0xf0)>>4);
            GPSStr[27] = 0x30+(((RadioCmdMsg[11].HexByte) & 0x0f));
            GPSStr[28] = ' ';
            GPSStr[29] = 'L';
            GPSStr[30] = 'o';
            GPSStr[31] = 'n';
            GPSStr[32] = 'g';
            GPSStr[33] = ' ';
            GPSStr[34] = 0x30+(((RadioCmdMsg[12].HexByte) & 0xf0)>>4);
            GPSStr[35] = 0x30+(((RadioCmdMsg[12].HexByte) & 0x0f));
            GPSStr[36] = 0x30+(((RadioCmdMsg[13].HexByte) & 0xf0)>>4);
            GPSStr[37] = '.';
            GPSStr[38] = 0x30+(((RadioCmdMsg[13].HexByte) & 0x0f));
            GPSStr[39] = 0x30+(((RadioCmdMsg[14].HexByte) & 0xf0)>>4);
            GPSStr[40] = '.';
            GPSStr[41] = 0x30+(((RadioCmdMsg[14].HexByte) & 0x0f));
            GPSStr[42] = 0x30+(((RadioCmdMsg[15].HexByte) & 0xf0)>>4);
            GPSStr[43] = 0x30+(((RadioCmdMsg[15].HexByte) & 0x0f));
            GPSStr[44] = 0;
         }

         //Display STATUS messages
         if(RadioCmdMsg[3].HexByte == 10)
         {
            sprintf(RxDataMessage, "Status: %s", StatusStr);
            DisplayPmrRxSerialData(RxDataMessage);
         }
         //Display STATUS + GPS messages
         else if(RadioCmdMsg[3].HexByte == 11)
         {
            sprintf(RxDataMessage, "Status: %s  GPS: %s", StatusStr, GPSStr);
            DisplayPmrRxSerialData(RxDataMessage);
         }
         //Display STATUS + JOB messages
         else if(RadioCmdMsg[3].HexByte == 12)
         {
            sprintf(RxDataMessage, "Status: %s  Job: %s", StatusStr, JobStr);
            DisplayPmrRxSerialData(RxDataMessage);
         }
         //Display STATUS + JOB + GPS messages
         else if(RadioCmdMsg[3].HexByte == 13)
         {
            sprintf(RxDataMessage, "Status: %s  Job: %s  GPS: %s", StatusStr, JobStr, GPSStr);
            DisplayPmrRxSerialData(RxDataMessage);
         }
         break;

    case ERROR_REPORT :
         MinimumMsgLength = 1;
         RadioCmdMsg[0].Message = "Error Report";

         if(RadioCmdMsg[1].HexByte == 0)
            RadioCmdMsg[1].Message = "Bad Command - Unknown Command Byte.";
         else if(RadioCmdMsg[1].HexByte == 1)
            RadioCmdMsg[1].Message = "Invalid parameter within command. ";
         else if(RadioCmdMsg[1].HexByte == 2)
            RadioCmdMsg[1].Message = "Bad Length (for Selcall String or Modem Data) ";
         else if(RadioCmdMsg[1].HexByte == 3)
            RadioCmdMsg[1].Message = "Cannot Transmit: Tx not allowed on channel. ";
         else if(RadioCmdMsg[1].HexByte == 4)
            RadioCmdMsg[1].Message = "Cannot Transmit: Carrier Inhibit ";
         else if(RadioCmdMsg[1].HexByte == 5)
            RadioCmdMsg[1].Message = "Cannot Transmit: Tx-Lockout Channel ";
         else if(RadioCmdMsg[1].HexByte == 6)
            RadioCmdMsg[1].Message = "Cannot Transmit: Community repeater in use ";
         else if(RadioCmdMsg[1].HexByte == 7)
            RadioCmdMsg[1].Message = "Cannot Transmit: Scanning & Mic in cradle. ";
         else if(RadioCmdMsg[1].HexByte == 8)
            RadioCmdMsg[1].Message = "Cannot Transmit: Selcall PTT Inhibit has been set. ";
         else if(RadioCmdMsg[1].HexByte == 9)
            RadioCmdMsg[1].Message = "Cannot Transmit: Cannot send selcall on non-selcall channel. ";
         else if(RadioCmdMsg[1].HexByte == 10)
            RadioCmdMsg[1].Message = "Data message failed, no ack from destination radio. ";
         else
            RadioCmdMsg[1].Message = "Undefined Error Number";

         break;

    default:
         MinimumMsgLength = 1;
         RadioCmdMsg[0].Message = "Unkown Command";
         for(j=0;j<CmdLength; j++)
            RadioCmdMsg[1+j].Message = "  ";

        break;
    }

    //Clear current string grid
    for(i=0; i<PMR_Form_Config->PMR_StrGrid_CmdDefinition->RowCount; i++)
    {
        PMR_Form_Config->PMR_StrGrid_CmdDefinition->Cells[0][i] = " ";
        PMR_Form_Config->PMR_StrGrid_CmdDefinition->Cells[1][i] = " ";
    }

    //copy bytes and difinitions to list
    char HexByteStr[3] = " ";

    for(i=0; i<CmdLength; i++)
    {
         highNibble = (RadioCmdMsg[i].HexByte & 0xF0) >> 4;
         lowNibble  = RadioCmdMsg[i].HexByte & 0x0F;

         if(highNibble <= 0x09)
            HexByteStr[0] = 48 + highNibble;
         else
            HexByteStr[0] = 65 + highNibble - 10;

         if(lowNibble <= 0x09)
            HexByteStr[1] = 48 + lowNibble;
         else
            HexByteStr[1] = 65 + lowNibble - 10;
         HexByteStr[2] = 0;

        PMR_Form_Config->PMR_StrGrid_CmdDefinition->Cells[0][i] = HexByteStr;
        PMR_Form_Config->PMR_StrGrid_CmdDefinition->Cells[1][i] = RadioCmdMsg[i].Message;
    }
//    for(i=MPT1327CmdLength; i<46; i++)
//    {
//        PMR_Form_Config->PMR_StrGrid_CmdDefinition->Cells[0][i] = " ";
//        PMR_Form_Config->PMR_StrGrid_CmdDefinition->Cells[1][i] = " ";
//    }
    PMR_Form_Config->PMR_StrGrid_CmdDefinition->Col = 0;
    PMR_Form_Config->PMR_StrGrid_CmdDefinition->Row = 0;
                              
};


